const std  = @import("std");
const dvui = @import("dvui");

pub const Task = struct {
    name: [:0]const u8,
    mark: bool,
};

pub fn gui(ctx: anytype) !void {
    var win = try dvui.floatingWindow(@src(), .{}, .{});
    defer win.deinit();

    try dvui.windowHeader("Today", "", null);

    var scrollArea = try dvui.scrollArea(@src(), .{}, .{
        .expand = .both,
    });
    defer scrollArea.deinit();

    var reorder = try dvui.reorder(@src(), .{
        .background = true,
        .expand     = .both,
    });
    defer reorder.deinit();

    var list = try dvui.box(@src(), .vertical, .{ .expand = .both, });
    defer list.deinit();

    const tasks = ctx.get_tasks();
    for (0.., tasks) |idx, task| {
        var reorderable = try reorder.reorderable(@src(), .{}, .{
            .id_extra = idx,
            .expand   = .horizontal,
        });
        defer reorderable.deinit();

        var hbox = try dvui.box(@src(), .horizontal, .{
            .expand     = .both,
            .background = true,
            .border     = dvui.Rect.all(1),
        });
        defer hbox.deinit();

        var mark: bool = task.mark;
        if (try dvui.checkbox(@src(), &mark, null, .{})) {
            ctx.mark(idx, mark);
        }

        try dvui.label(@src(), "{s}", .{ task.name, }, .{});

        _ = try dvui.ReorderWidget.draggable(@src(), .{ .reorderable = reorderable, }, .{
            .expand      = .vertical,
            .gravity_x   = 1.0,
            .gravity_y   = 0.5,
        });
    }

    try dvui.labelNoFmt(@src(), "no tasks anymore :)", .{ .gravity_x = 0.5 });
}

pub const Mock = struct {
    const Self     = @This();
    const TaskList = std.ArrayList(Task);

    tasks: TaskList,

    pub fn init(alloc: std.mem.Allocator) !Self {
        var tasks = TaskList.init(alloc);
        errdefer tasks.deinit();

        try tasks.append(Task { .name = "helloworld", .mark = true, });
        try tasks.append(Task { .name = "goodbye", .mark = false, });

        return Mock {
            .tasks = tasks,
        };
    }
    pub fn deinit(self: *Self) void {
        self.tasks.deinit();
    }

    fn mark(self: *Self, idx: usize, check: bool) void {
        self.tasks.items[idx].mark = check;
    }
    fn get_tasks(self: *const Self) []const Task {
        return self.tasks.items;
    }
};
